@@ -7,12 +7,14 @@ use crate::paste::{Paste, PasteType};
 use crate::util;
 use actix_files::NamedFile;
 use actix_multipart::Multipart;
+use actix_web::client::Client;
 use actix_web::{error, get, post, web, Error, HttpRequest, HttpResponse, Responder};
 use byte_unit::Byte;
 use futures_util::stream::StreamExt;
 use std::convert::TryFrom;
 use std::env;
 use std::fs;
+use std::sync::{Arc, Mutex};
 
 /// Shows the landing page.
 #[get("/")]
@@ -27,8 +29,9 @@ async fn index() -> impl Responder {
 async fn serve(
     request: HttpRequest,
     file: web::Path<String>,
-    config: web::Data<Config>,
+    config: web::Data<Arc<Mutex<Config>>>,
 ) -> Result<HttpResponse, Error> {
+    let config = config.lock().expect("cannot acquire config");
     let path = config.server.upload_path.join(&*file);
     let mut path = util::glob_match_file(path)?;
     let mut paste_type = PasteType::File;
@@ -37,7 +40,7 @@ async fn serve(
             let alt_path = type_.get_path(&config.server.upload_path).join(&*file);
             let alt_path = util::glob_match_file(alt_path)?;
             if alt_path.exists()
-                || path.file_name().map(|v| v.to_str()).flatten() == Some(&type_.get_dir())
+                || path.file_name().and_then(|v| v.to_str()) == Some(&type_.get_dir())
             {
                 path = alt_path;
                 paste_type = *type_;
@@ -46,7 +49,7 @@ async fn serve(
         }
     }
     match paste_type {
-        PasteType::File | PasteType::Oneshot => {
+        PasteType::File | PasteType::RemoteFile | PasteType::Oneshot => {
             let response = NamedFile::open(&path)
                 .map_err(|_| error::ErrorNotFound("file is not found or expired :("))?
                 .disable_content_disposition()
@@ -79,13 +82,15 @@ async fn serve(
 async fn upload(
     request: HttpRequest,
     mut payload: Multipart,
-    config: web::Data<Config>,
+    client: web::Data<Client>,
+    config: web::Data<Arc<Mutex<Config>>>,
 ) -> Result<HttpResponse, Error> {
     let connection = request.connection_info();
     let host = connection.remote_addr().unwrap_or("unknown host");
     auth::check(host, request.headers(), env::var("AUTH_TOKEN").ok())?;
     let expiry_date = header::parse_expiry_date(request.headers())?;
     let mut urls: Vec<String> = Vec::new();
+    let config = config.lock().expect("cannot acquire config");
     while let Some(item) = payload.next().await {
         let mut field = item?;
         let content = ContentDisposition::try_from(field.content_disposition())?;
@@ -94,7 +99,7 @@ async fn upload(
             while let Some(chunk) = field.next().await {
                 bytes.append(&mut chunk?.to_vec());
                 if bytes.len() as u128 > config.server.max_content_length.get_bytes() {
-                    log::warn!("upload rejected for {}", host);
+                    log::warn!("Upload rejected for {}", host);
                     return Err(error::ErrorPayloadTooLarge("upload limit exceeded"));
                 }
             }
@@ -102,7 +107,11 @@ async fn upload(
                 log::warn!("{} sent zero bytes", host);
                 return Err(error::ErrorBadRequest("invalid file size"));
             }
-            if paste_type != PasteType::Oneshot && !config.paste.duplicate_files.unwrap_or(true) {
+            if paste_type != PasteType::Oneshot
+                && paste_type != PasteType::RemoteFile
+                && expiry_date.is_none()
+                && !config.paste.duplicate_files.unwrap_or(true)
+            {
                 let bytes_checksum = util::sha256_digest(&*bytes)?;
                 if let Some(file) = Directory::try_from(config.server.upload_path.as_path())?
                     .get_file(bytes_checksum)
@@ -120,18 +129,27 @@ async fn upload(
                     continue;
                 }
             }
-            let bytes_unit = Byte::from_bytes(bytes.len() as u128).get_appropriate_unit(false);
-            let paste = Paste {
+            let mut paste = Paste {
                 data: bytes.to_vec(),
                 type_: paste_type,
             };
-            let file_name = match paste_type {
+            let file_name = match paste.type_ {
                 PasteType::File | PasteType::Oneshot => {
                     paste.store_file(content.get_file_name()?, expiry_date, &config)?
                 }
+                PasteType::RemoteFile => {
+                    paste
+                        .store_remote_file(expiry_date, &client, &config)
+                        .await?
+                }
                 PasteType::Url => paste.store_url(expiry_date, &config)?,
             };
-            log::info!("{} ({}) is uploaded from {}", file_name, bytes_unit, host);
+            log::info!(
+                "{} ({}) is uploaded from {}",
+                file_name,
+                Byte::from_bytes(paste.data.len() as u128).get_appropriate_unit(false),
+                host
+            );
             urls.push(format!(
                 "{}://{}/{}\n",
                 connection.scheme(),
